/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 * Henrik Gemal http://gemal.dk
 *
 * Portions created by the Initial Developer are Copyright (C) 2001-2004
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var LaunchyContext = function() {}

LaunchyContext.prototype.findApp = function(reg, reg_sub, reg_name, app_name, app_exe, app_gecko) {

	// reg values:
	// 0: HKEY_CLASSES_ROOT
	// 1: HKEY_CURRENT_CONFIG
	// 2: HKEY_CURRENT_USER
	// 3: HKEY_LOCAL_MACHINE
	// 4: HKEY_USERS

	var reg_type = new Array("HKEY_CLASSES_ROOT", "HKEY_CURRENT_CONFIG", "HKEY_CURRENT_USER", "HKEY_LOCAL_MACHINE", "HKEY_USERS");
	launchyShared.trace("Looking for <" + app_name + "> in <" + reg_type[reg] + "> at <" + reg_sub + (reg_name?"\\"+reg_name:"") + ">" + (parseInt(app_gecko)?" and it's a Gecko":""));
	try {
		var tmp = LaunchyContext.winhooks.getRegistryEntry(reg, reg_sub, reg_name);
		var tmp_gecko;
		if (app_gecko && tmp) {
			launchyShared.trace("Found Gecko version <" + tmp + ">");
			tmp_gecko = tmp;
			tmp = LaunchyContext.winhooks.getRegistryEntry(reg, reg_sub + "\\" + tmp + "\\Main", "PathToExe");
		}
	} catch(ex) {
		launchyShared.trace("Unable to find " + app_name + " due to " + ex);
		return false;
	}
	if (tmp) {
		var re=/\\\\/;
		tmp = tmp.replace(re, "\\");
		re=/\\$/;
		tmp = tmp.replace(re, "");

		launchyShared.trace("<" + app_name + "> registry entry found <" + tmp + ">");
		if (app_exe) {
			tmp += "\\" + app_exe;
		}

		re=/\"(.*?)\" ?/;
		var x = tmp.match(re);
		if (x && x[1]) {
			tmp = x[1];
		}

		if (LaunchyContext.fu.exists(tmp)) {
			var f = new File(tmp);
			if (f.isFile() && f.isExec()) {
				launchyShared.trace(app_name + (app_gecko ? " <" + tmp_gecko + ">" : "") + " is installed at <" + tmp + ">");
				if (app_gecko && tmp.toLowerCase().indexOf(LaunchyContext.progdir) != -1) {
					launchyShared.trace("Hide " + app_name + " since it's the running client");
					return false;
				}
			} else {
				launchyShared.trace("File <" + tmp + "> isn't a file or isn't executable");
				return false;
			}
			return tmp;
		} else {
			launchyShared.trace(app_name + " isn't at: <" + tmp + ">");
			return false;
		}
	} else {
		launchyShared.trace(app_name + " isn't installed or undetected");
		return false;
	}
}

LaunchyContext.prototype.createApp = function(app_name, app_exe, app_type, app_args) {
	var app = new Array();
	app.name = app_name;
	app.exe = app_exe;
	app.type = app_type;
	app.args = app_args;
	return app;
}

LaunchyContext.prototype.addApp = function(reg, reg_sub, reg_name, app_name, app_exe, app_type, app_args, app_gecko) {
	// Check for dupes
	for (var i = 0; LaunchyContext.apps.length > i; i++) {
		if (LaunchyContext.apps[i].name == app_name) {
			launchyShared.trace("<" + app_name + "> is already detected");
			return;
		}
	}
	var tmp = launchyContext.findApp(reg, reg_sub, reg_name, app_name, app_exe, app_gecko);
	if (tmp) {
		LaunchyContext.apps[LaunchyContext.apps.length] = launchyContext.createApp(app_name, tmp, app_type, app_args);
	}
}

LaunchyContext.prototype.loadApps = function() {
	var chromedir = new Dir(LaunchyContext.du.getUserChromeDir());
	var file = chromedir.append("launchy.rdf");
	if (LaunchyContext.fu.exists(file)) {
		launchyShared.trace("Loading applications from RDF <" + file + ">");
		var rdf = new RDFFile(file);
		try {
			var conts = rdf.getAllContainers();
			for (var i=0; i<conts.length; i++) {
				var list = conts[i].getSubNodes();
				for (i=0; i<list.length; i++) {
					var subject = list[i].getSubject();
					subject.match(/urn:launchy:app:(.*)/);
					launchyContext.addApp(list[i].getAttribute("http://launchy.mozdev.org/app#registry_type"), list[i].getAttribute("http://launchy.mozdev.org/app#registry_key"), list[i].getAttribute("http://launchy.mozdev.org/app#registry_entry"), RegExp.$1, list[i].getAttribute("http://launchy.mozdev.org/app#exe"), list[i].getAttribute("http://launchy.mozdev.org/app#type"), list[i].getAttribute("http://launchy.mozdev.org/app#arguments"), list[i].getAttribute("http://launchy.mozdev.org/app#gecko"));
				}
			}
		} catch(ex) {
			launchyShared.trace("Error loading applications: " + ex);
		}
	}
}

LaunchyContext.prototype.doInit = function() {
	// did Launchy already load?
	var ass = Components.classes["@mozilla.org/appshell/appShellService;1"].getService(Components.interfaces.nsIAppShellService);
	var w = ass.hiddenDOMWindow;
	if (typeof(w.launchy) == "undefined") {
		LaunchyContext.version = "2.3.0";
		w.launchy = LaunchyContext;
		var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		var pref = prefService.getBranch(null);
		try {
			LaunchyContext.dev = pref.getBoolPref("extensions.launchy.developer");
			launchyShared.trace("Developer information is on");
			launchyShared.trace("Launchy version <" + LaunchyContext.version + "> installed");
		} catch(ex) {
		}

		LaunchyContext.launchy = true;
		try {
			include(jslib_fileutils);
			include(jslib_dirutils);
			include(jslib_dir);
			include(jslib_rdf);
			include(jslib_rdffile);
			include(jslib_rdfcontainer);
			include(jslib_rdfresource);
		} catch(ex) {
			launchyShared.trace("JSLib isn't installed! Launchy will be disabled!");
			LaunchyContext.launchy = false;
			return;
		}

		launchyShared.trace("JSLib version <" + JS_LIB_VERSION + "> installed");

		LaunchyContext.fu=new FileUtils();
		LaunchyContext.du=new DirUtils();
		LaunchyContext.progdir = LaunchyContext.du.getCurProcDir().toLowerCase();
		launchyShared.trace("Program directory is <" + LaunchyContext.progdir + ">");

		if ("@mozilla.org/winhooks;1" in Components.classes) {
			try {
				LaunchyContext.winhooks = Components.classes["@mozilla.org/winhooks;1"].getService(Components.interfaces.nsIWindowsRegistry);
			} catch(ex) {
				launchyShared.trace("Unable to get winhooks service!");
			}
		} else if ("@mozilla.org/browser/shell-service;1" in Components.classes) {
			try {
				LaunchyContext.winhooks = Components.classes["@mozilla.org/browser/shell-service;1"].getService(Components.interfaces.nsIWindowsShellService);
			} catch(ex) {
				launchyShared.trace("Unable to get browser shell service!");
			}
		} else {
			launchyShared.trace("Unable to get Components");
			return;
		}
		if (typeof(LaunchyContext.winhooks) == "undefined" || !("getRegistryEntry" in LaunchyContext.winhooks)) {
			launchyShared.trace("getRegistryEntry is not available!");
			return;
		}

		LaunchyContext.apps = new Array();
		launchyContext.loadApps();
		launchyShared.trace("Loading applications from JavaScript");

		// Browsers
		launchyContext.addApp(3, "Software\\mozilla.org\\Mozilla", "CurrentVersion", "Mozilla", "", 1, "", 1);
		launchyContext.addApp(3, "Software\\mozilla.org\\Mozilla Firebird", "CurrentVersion", "Mozilla Firebird", "", 1, "", 1);
		launchyContext.addApp(3, "Software\\mozilla.org\\Mozilla Firefox", "CurrentVersion", "Mozilla Firefox", "", 1, "", 1);
		launchyContext.addApp(3, "Software\\mozilla.org\\Netscape", "CurrentVersion", "Netscape", "", 1, "", 1);
		launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\Opera.exe", "", "Opera", "", 1, "", 0);
		launchyContext.addApp(2, "Software\\K-Meleon\\K-Meleon\\General", "InstallDir", "K-Meleon", "k-meleon.exe", 1, "", 0);
		launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\MYIE2", "DisplayIcon", "MyIE2", "", 1, "", 0);
		launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\IEXPLORE.EXE", "", "Internet Explorer", "", 1, "", 0);
		launchyContext.addApp(2, "Software\\Siber Systems\\RoboForm", "_TBP_", "Avant Browser", "", 1, "", 0);
		launchyContext.addApp(2, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\Crazy Browser_is1", "DisplayIcon", "Crazy Browser", "", 1, "", 0);

		// Mail clients
		launchyContext.addApp(3, "Software\\mozilla.org\\Mozilla", "CurrentVersion", "Mozilla Mail", "", 2, ["-compose"], 1);
		launchyContext.addApp(3, "Software\\mozilla.org\\Mozilla Thunderbird", "CurrentVersion", "Mozilla Thunderbird", "", 2, ["-compose"], 1);
		launchyContext.addApp(2, "Software\\Pegasus Mail\\BaseDir", "", "Pegasus Mail", "wsendto.exe", 2, "", 0);
		launchyContext.addApp(3, "Software\\Lotus\\Notes", "Path", "Lotus Notes", "notes.exe", 2, "", 0);
		launchyContext.addApp(2, "Software\\Qualcomm\\Eudora\\CommandLine", "current", "Eudora", "", 2, "", 0);
		launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\OUTLOOK.EXE", "", "Outlook", "", 2, ["-c", "IPM.Note", "/m"], 0);
		launchyContext.addApp(2, "Software\\RIT\\The Bat!", "EXE path", "The Bat!", "", 2, "", 0);
		launchyContext.addApp(3, "Software\\IncrediMail", "InstallationPath", "IncrediMail", "bin\\IncMail.exe", 2, ["/c", "/n", "/mailto:#1"] , 0);
		launchyContext.addApp(3, "Software\\MCS\\Calypso3", "Path", "Calypso", "Calypso.exe", 2, ["/MAILTO"] , 0);

		// Special case applications
		var tmp = launchyContext.findApp(3, "Software\\Microsoft\\WAB\\Export\\CSV", "DLL", "Outlook Express");
		if (tmp) {
			var re=/wabimp\.dll/i;
			tmp = tmp.replace(re, "msimn.exe");
			LaunchyContext.apps[LaunchyContext.apps.length] = launchyContext.createApp("Outlook Express", tmp, 2, ["/mailurl:#1"]);
		}

		// Media clients
		launchyContext.addApp(2, "Software\\BST\\bsplayerv1", "AppPath", "BSPlayer", "", 3, "", 0);
		launchyContext.addApp(3, "Software\\Clients\\Media\\iTunes\\shell\\open\\command", "", "iTunes", "", 3, ["/url"], 0);
		launchyContext.addApp(2, "Software\\Winamp", "", "Winamp", "winamp.exe", 3, "", 0);
		launchyContext.addApp(3, "Software\\Microsoft\\MediaPlayer", "Installation Directory", "Windows Media Player", "wmplayer.exe", 3, "", 0);

		// FTP clients
		launchyContext.addApp(2, "Software\\Ghisler\\Total Commander", "InstallDir", "Total Commander", "totalcmd.exe", 4, ["/O"], 0);
		launchyContext.addApp(2, "Software\\UltraFXP", "", "UltraFXP", "ultrafxp.exe", 4, "", 0);
		launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\wsftppro.exe", "", "WS_FTP Pro", "", 4, "", 0);
		launchyContext.addApp(3, "Software\\SmartFTP", "Install Directory", "SmartFTP", "smartftp.exe", 4, "", 0);
		launchyContext.addApp(2, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\BulletProof FTP_is1", "DisplayIcon", "BulletProof FTP", "", 4, "", 0);

		// Download clients
		launchyContext.addApp(2, "Software\\HeadLight\\GetRight\\Config", "InstallDir", "GetRight", "getright.exe", 5, "", 0);
		launchyContext.addApp(2, "Software\\Cronosoft\\LeechGet\\Console", "LeechGetPath", "LeechGet", "", 5, ["-wizard"], 0);
		launchyContext.addApp(0, "TypeLib\\{3F9A5301-9E5A-11D3-A288-0000E80E2EDE}\\1.0\\HELPDIR", "", "Mass Downloader", "massdown.exe", 5, "", 0);
		launchyContext.addApp(2, "Software\\Star Downloader", "Path", "Star Downloader", "", 5, "", 0);
		launchyContext.addApp(2, "Software\\DownloadManager", "ExePath", "Internet Download Manager", "", 5, ["/d"], 0);
		launchyContext.addApp(0, "TypeLib\\{1537E849-0200-11D2-8059-000000000000}\\1.0\\HELPDIR", "", "ReGet Deluxe", "regetdx.exe", 5, "", 0);
		launchyContext.addApp(0, "bittorrent\\shell\\open\\command", "", "BitTorrent", "", 5, ["--url"], 0);
		launchyContext.addApp(2, "Software\\JetCar\\JetCar\\General", "AppPath", "FlashGet", "", 5, "", 0);
		launchyContext.addApp(2, "Software\\Radiate\\Go!Zilla", "", "Go!Zilla", "", 5, "", 0);

		w.launchy = LaunchyContext;
	} else {
		LaunchyContext = w.launchy;
	}
}

LaunchyContext.prototype.doContext = function(i) {
	if (!LaunchyContext.launchy) {
		document.getElementById(i + "-disabled").hidden = false;
		return;
	}
	if (typeof(LaunchyContext.apps) == "undefined" || !LaunchyContext.apps.length) {
		document.getElementById(i + "-none").hidden = false;
		return;
	}

	var menuid = document.getElementById(i);
  var children = menuid.childNodes;
  for (var j = 0; j < children.length; j++) {
		menuid.removeChild(children[j]);
	}

	var label;
	var tooltip;

	// right click on page
	if (!gContextMenu.onLink && !gContextMenu.onMailtoLink) {
		label = document.getElementById("bundle_launchy").getString("launchy-page-label");
		tooltip = document.getElementById("bundle_launchy").getString("launchy-page-tooltip");

		var flabel;
		var ftooltip;
		if (gContextMenu.inFrame) {
			flabel = document.getElementById("bundle_launchy").getString("launchy-frame-label");
			ftooltip = document.getElementById("bundle_launchy").getString("launchy-frame-tooltip");
		}
		for (i = 0; LaunchyContext.apps.length > i; i++) {
			if (LaunchyContext.apps[i].type == 1) {
				var menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\");");
				menuid.appendChild(menuitem);
				if (gContextMenu.inFrame) {
					menuitem = document.createElement("menuitem");
					menuitem.setAttribute("label", launchyShared.replaceInsert(flabel, 1, LaunchyContext.apps[i].name));
					menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(ftooltip, 1, LaunchyContext.apps[i].name));
					menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 0, 1);");
					menuid.appendChild(menuitem);
				}
			}
		}
	}

	// right click on link
	else if (gContextMenu.onLink && !gContextMenu.onMailtoLink) {
		label = document.getElementById("bundle_launchy").getString("launchy-link-label");
		tooltip = document.getElementById("bundle_launchy").getString("launchy-link-tooltip");
		var seperator = 0;
		for (i = 0; LaunchyContext.apps.length > i; i++) {
			if (LaunchyContext.apps[i].type == 1) {
				menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 1);");
				menuid.appendChild(menuitem);
				seperator = 1;
			}
		}
		for (i = 0; LaunchyContext.apps.length > i; i++) {
			if (LaunchyContext.apps[i].type == 3) {
				if (seperator == 1) {
					menuitem = document.createElement("menuseparator");
					menuid.appendChild(menuitem);
					seperator++;
				}
				menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 1);");
				menuid.appendChild(menuitem);
			}
		}
		var url = "" + gContextMenu.link;
		var re=/^(http:\/\/ftp|ftp:\/\/)/i;
		if (url.match(re)) {
			url = url.replace(/^http:\/\//i, "ftp:\/\/");
			label = document.getElementById("bundle_launchy").getString("launchy-ftp-label");
			tooltip = document.getElementById("bundle_launchy").getString("launchy-ftp-tooltip");
			for (i = 0; LaunchyContext.apps.length > i; i++) {
				if (LaunchyContext.apps[i].type == 4) {
					if (seperator == 2) {
						menuitem = document.createElement("menuseparator");
						menuid.appendChild(menuitem);
						seperator++;
					}
					menuitem = document.createElement("menuitem");
					menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
					menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
					menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 1, 0, \"" + url + "\");");
					menuid.appendChild(menuitem);
				}
			}
		}
		label = document.getElementById("bundle_launchy").getString("launchy-download-label");
		tooltip = document.getElementById("bundle_launchy").getString("launchy-download-tooltip");
		for (i = 0; LaunchyContext.apps.length > i; i++) {
			if (LaunchyContext.apps[i].type == 5) {
				if (seperator == 3) {
					menuitem = document.createElement("menuseparator");
					menuid.appendChild(menuitem);
					seperator++;
				}
				menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 1);");
				menuid.appendChild(menuitem);
			}
		}
	}

	// right click on mailto link
	else if (gContextMenu.onMailtoLink) {
		label = document.getElementById("bundle_launchy").getString("launchy-mail-label");
		tooltip = document.getElementById("bundle_launchy").getString("launchy-mail-tooltip");
		for (i = 0; LaunchyContext.apps.length > i; i++) {
			if (LaunchyContext.apps[i].type == 2) {
				menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 1);");
				menuid.appendChild(menuitem);
			}
		}
	}
}

LaunchyContext.prototype.doApp = function(app_entry, on_link, on_frame, on_url) {
	var app_exe = LaunchyContext.apps[app_entry].exe;

	var url;
	if (on_url) {
		url = on_url;
	} else if (on_link) {
		url = gContextMenu.link;
	} else {
		if (on_frame) {
			url = window._content.location.href;
		} else {
			url = document.commandDispatcher.focusedWindow.document.location.href;
		}
	}

	var xargs = new Array();
	if (LaunchyContext.apps[app_entry].args)
		xargs = LaunchyContext.apps[app_entry].args.join(',').split(',');

	var u = false;
	for (var i=0; i < xargs.length; i++) {
		var tmp = launchyShared.replaceInsert(xargs[i], 1, url)
		if (tmp != xargs[i]) {
			xargs[i] = tmp;
			u = true;
		}
	}
	if (!u)
		xargs[xargs.length] = url;

	launchyShared.trace("About to run:\n" + app_exe + " " + xargs.join(" "));
	LaunchyContext.fu.run(app_exe, xargs);
}

// create the object
const launchyContext = new LaunchyContext();
launchyContext.doInit();
